"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
class ConfigLoader {
    constructor(configClass) {
        this.cache = new Map();
        this.configClass = configClass;
    }
    flush(filename) {
        if (filename) {
            this.cache.delete(filename);
        }
        else {
            this.cache.clear();
        }
    }
    fromTarget(filename) {
        if (filename === "inline") {
            return this.configClass.empty();
        }
        if (this.cache.has(filename)) {
            return this.cache.get(filename);
        }
        let current = path_1.default.resolve(path_1.default.dirname(filename));
        let config = this.configClass.empty();
        for (;;) {
            const search = path_1.default.join(current, ".htmlvalidate.json");
            if (fs_1.default.existsSync(search)) {
                const local = this.configClass.fromFile(search);
                config = local.merge(config);
            }
            const child = current;
            current = path_1.default.dirname(current);
            if (current === child) {
                break;
            }
        }
        this.cache.set(filename, config);
        return config;
    }
}
exports.ConfigLoader = ConfigLoader;
