"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const nodetype_1 = require("./nodetype");
const DOCUMENT_NODE_NAME = "#document";
let counter = 0;
function reset() {
    counter = 0;
}
exports.reset = reset;
class DOMNode {
    constructor(nodeType, nodeName, location) {
        this.nodeType = nodeType;
        this.nodeName = nodeName || DOCUMENT_NODE_NAME;
        this.location = location;
        this.disabledRules = new Set();
        this.childNodes = [];
        this.unique = counter++;
    }
    get textContent() {
        return this.childNodes.map(node => node.textContent).join("");
    }
    append(node) {
        this.childNodes.push(node);
    }
    isRootElement() {
        return this.nodeType === nodetype_1.NodeType.DOCUMENT_NODE;
    }
    get firstChild() {
        return this.childNodes[0] || null;
    }
    get lastChild() {
        return this.childNodes[this.childNodes.length - 1] || null;
    }
    disableRule(ruleId) {
        this.disabledRules.add(ruleId);
    }
    enableRule(ruleId) {
        this.disabledRules.delete(ruleId);
    }
    ruleEnabled(ruleId) {
        return !this.disabledRules.has(ruleId);
    }
}
exports.DOMNode = DOMNode;
