import { Location } from "../context";
import { Token } from "../lexer";
import { MetaElement, MetaTable } from "../meta";
import { Attribute } from "./attribute";
import { DOMNode } from "./domnode";
import { DOMTokenList } from "./domtokenlist";
import { DynamicValue } from "./dynamic-value";
export declare enum NodeClosed {
    Open = 0,
    EndTag = 1,
    VoidOmitted = 2,
    VoidSelfClosed = 3,
    ImplicitClosed = 4
}
export declare class HtmlElement extends DOMNode {
    readonly tagName: string;
    readonly meta: MetaElement;
    readonly parent: HtmlElement;
    readonly voidElement: boolean;
    readonly depth: number;
    closed: NodeClosed;
    protected readonly attr: {
        [key: string]: Attribute[];
    };
    constructor(tagName: string, parent?: HtmlElement, closed?: NodeClosed, meta?: MetaElement, location?: Location);
    static rootNode(location: Location): HtmlElement;
    static fromTokens(startToken: Token, endToken: Token, parent: HtmlElement, metaTable: MetaTable): HtmlElement;
    readonly childElements: HtmlElement[];
    is(tagName: string): boolean;
    setAttribute(key: string, value: string | DynamicValue, keyLocation: Location, valueLocation: Location, originalAttribute?: string): void;
    readonly attributes: Attribute[];
    hasAttribute(key: string): boolean;
    getAttribute(key: string): Attribute;
    getAttribute(key: string, all: true): Attribute[];
    getAttributeValue(key: string): string;
    appendText(text: string | DynamicValue, location?: Location): void;
    readonly classList: DOMTokenList;
    readonly id: string;
    readonly siblings: HtmlElement[];
    readonly previousSibling: HtmlElement;
    readonly nextSibling: HtmlElement;
    getElementsByTagName(tagName: string): HtmlElement[];
    querySelector(selector: string): HtmlElement;
    querySelectorAll(selector: string): HtmlElement[];
    private querySelectorImpl;
    visitDepthFirst(callback: (node: HtmlElement) => void): void;
    someChildren(callback: (node: HtmlElement) => boolean): boolean;
    everyChildren(callback: (node: HtmlElement) => boolean): boolean;
    find(callback: (node: HtmlElement) => boolean): HtmlElement;
}
