import { Config, Severity } from "../config";
import { Source } from "../context";
import { Parser } from "../parser";
import { Report, Reporter } from "../reporter";
import { Rule, RuleConstructor, RuleDocumentation, RuleOptions } from "../rule";
export interface EventDump {
    event: string;
    data: any;
}
export interface TokenDump {
    token: string;
    data: string;
    location: string;
}
export declare class Engine<T extends Parser = Parser> {
    protected report: Reporter;
    protected config: Config;
    protected ParserClass: new (config: Config) => T;
    protected availableRules: {
        [key: string]: RuleConstructor;
    };
    constructor(config: Config, ParserClass: new (config: Config) => T);
    lint(sources: Source[]): Report;
    dumpEvents(source: Source[]): EventDump[];
    dumpTokens(source: Source[]): TokenDump[];
    dumpTree(source: Source[]): string[];
    getRuleDocumentation(ruleId: string, context?: any): RuleDocumentation;
    private processDirective;
    private processEnableDirective;
    private processDisableDirective;
    private processDisableBlockDirective;
    private processDisableNextDirective;
    protected initPlugins(config: Config): {
        availableRules: {
            [key: string]: RuleConstructor;
        };
    };
    protected initRules(config: Config): {
        [key: string]: RuleConstructor;
    };
    protected setupPlugins(source: Source, config: Config, parser: Parser): {
        rules: {
            [key: string]: Rule;
        };
    };
    protected setupRules(config: Config, parser: Parser): {
        [key: string]: Rule;
    };
    protected loadRule(ruleId: string, severity: Severity, options: any, parser: Parser, report: Reporter): Rule;
    protected instantiateRule(name: string, options: RuleOptions): Rule;
    protected requireRule(name: string, options: RuleOptions): any;
    private missingRule;
    private reportError;
}
