import { Config } from "../config";
import { Location, Source } from "../context";
import { DOMTree, HtmlElement } from "../dom";
import { ElementReadyEvent, EventCallback, EventHandler } from "../event";
import { AttributeEvent, ConditionalEvent, DirectiveEvent, DoctypeEvent, DOMReadyEvent, Event, TagCloseEvent, TagOpenEvent, WhitespaceEvent } from "../event";
import { Token, TokenStream, TokenType } from "../lexer";
export declare class Parser {
    private readonly event;
    private readonly metaTable;
    private dom;
    constructor(config: Config);
    parseHtml(source: string | Source): DOMTree;
    private closeOptional;
    protected consumeTag(source: Source, startToken: Token, tokenStream: TokenStream): void;
    protected closeElement(source: Source, node: HtmlElement, active: HtmlElement, location: Location): void;
    protected discardForeignBody(source: Source, foreignTagName: string, tokenStream: TokenStream): void;
    protected consumeAttribute(source: Source, node: HtmlElement, token: Token, next?: Token): void;
    private getAttributeValueLocation;
    protected consumeDirective(token: Token): void;
    protected consumeDoctype(startToken: Token, tokenStream: TokenStream): void;
    protected consumeUntil(tokenStream: TokenStream, search: TokenType): IterableIterator<Token>;
    private next;
    on(event: string, listener: EventCallback): () => void;
    once(event: string, listener: EventCallback): () => void;
    defer(cb: () => void): void;
    protected trigger(event: "tag:open", data: TagOpenEvent): void;
    protected trigger(event: "tag:close", data: TagCloseEvent): void;
    protected trigger(event: "element:ready", data: ElementReadyEvent): void;
    protected trigger(event: "dom:load", data: Event): void;
    protected trigger(event: "dom:ready", data: DOMReadyEvent): void;
    protected trigger(event: "doctype", data: DoctypeEvent): void;
    protected trigger(event: "attr", data: AttributeEvent): void;
    protected trigger(event: "whitespace", data: WhitespaceEvent): void;
    protected trigger(event: "conditional", data: ConditionalEvent): void;
    protected trigger(event: "directive", data: DirectiveEvent): void;
    getEventHandler(): EventHandler;
    private appendText;
    private closeTree;
}
