"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rule_1 = require("../../rule");
class H67 extends rule_1.Rule {
    documentation() {
        return {
            description: "A decorative image cannot have a title attribute. Either remove `title` or add a descriptive `alt` text.",
            url: rule_1.ruleDocumentationUrl(__filename),
        };
    }
    constructor(options) {
        super(options);
        this.name = "WCAG/H67";
    }
    setup() {
        this.on("tag:close", (event) => {
            const node = event.target;
            if (!node || node.tagName !== "img") {
                return;
            }
            const title = node.getAttribute("title");
            if (!title || title.value === "") {
                return;
            }
            const alt = node.getAttributeValue("alt");
            if (alt && alt !== "") {
                return;
            }
            this.report(node, "<img> with empty alt text cannot have title attribute", title.keyLocation);
        });
    }
}
module.exports = H67;
